@@ -2,13 +2,13 @@
 
   readline.c - GNU Readline module
 
-  $Author: nagachika $
+  $Author: nobu $
   created at: Wed Jan 20 13:59:32 JST 1999
 
   Copyright (C) 1997-2008  Shugo Maeda
   Copyright (C) 2008-2013  Kouji Takao
 
-  $Id: readline.c 61456 2017-12-24 20:06:32Z nagachika $
+  $Id: readline.c 60071 2017-09-30 08:35:23Z nobu $
 
   Contact:
    - Kouji Takao <kouji dot takao at gmail dot com> (current maintainer)
@@ -165,24 +165,25 @@ getc_body(struct getc_struct *p)
 #if defined(_WIN32)
     {
         INPUT_RECORD ir;
-        int n;
+        DWORD n;
         static int prior_key = '0';
         for (;;) {
+            HANDLE h;
             if (prior_key > 0xff) {
                 prior_key = rl_getc(p->input);
                 return prior_key;
             }
-            if (PeekConsoleInput((HANDLE)_get_osfhandle(p->fd), &ir, 1, &n)) {
+            h = (HANDLE)_get_osfhandle(p->fd);
+            if (PeekConsoleInput(h, &ir, 1, &n)) {
                 if (n == 1) {
                     if (ir.EventType == KEY_EVENT && ir.Event.KeyEvent.bKeyDown) {
                         prior_key = rl_getc(p->input);
                         return prior_key;
                     } else {
-                        ReadConsoleInput((HANDLE)_get_osfhandle(p->fd), &ir, 1, &n);
+                        ReadConsoleInput(h, &ir, 1, &n);
                     }
                 } else {
-                    HANDLE h = (HANDLE)_get_osfhandle(p->fd);
-                    rb_w32_wait_events(&h, 1, INFINITE);
+                    rb_w32_wait_events_blocking(&h, 1, INFINITE);
                 }
             } else {
                 break;
@@ -2077,7 +2078,7 @@ Init_readline(void)
 
     rl_attempted_completion_function = readline_attempted_completion_function;
 #if defined(HAVE_RL_PRE_INPUT_HOOK)
-    rl_pre_input_hook = readline_pre_input_hook;
+    rl_pre_input_hook = (rl_hook_func_t *)readline_pre_input_hook;
 #endif
 #if defined HAVE_RL_CHAR_IS_QUOTED_P
     rl_char_is_quoted_p = &readline_char_is_quoted;
