@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require 'rubygems/command'
 require 'rubygems/package'
 require 'rubygems/installer'
@@ -21,6 +22,11 @@ class Gem::Commands::PristineCommand < Gem::Command
       options[:all] = value
     end
 
+    add_option('--skip=gem_name',
+               'used on --all, skip if name == gem_name') do |value, options|
+      options[:skip] = value
+    end
+
     add_option('--[no-]extensions',
                'Restore gems with extensions',
                'in addition to regular gems') do |value, options|
@@ -86,8 +92,8 @@ extensions will be restored.
                 spec.extensions and not spec.extensions.empty?
               end
             else
-              get_all_gem_names.map do |gem_name|
-                Gem::Specification.find_all_by_name gem_name, options[:version]
+              get_all_gem_names.sort.map do |gem_name|
+                Gem::Specification.find_all_by_name(gem_name, options[:version]).reverse
               end.flatten
             end
 
@@ -109,19 +115,24 @@ extensions will be restored.
         next
       end
 
+      if spec.name == options[:skip]
+        say "Skipped #{spec.full_name}, it was given through options"
+        next
+      end
+
       if spec.bundled_gem_in_old_ruby?
         say "Skipped #{spec.full_name}, it is bundled with old Ruby"
         next
       end
 
-      unless spec.extensions.empty? or options[:extensions] then
+      unless spec.extensions.empty? or options[:extensions] or options[:only_executables] then
         say "Skipped #{spec.full_name}, it needs to compile an extension"
         next
       end
 
       gem = spec.cache_file
 
-      unless File.exist? gem then
+      unless File.exist? gem or options[:only_executables] then
         require 'rubygems/remote_fetcher'
 
         say "Cached gem for #{spec.full_name} not found, attempting to fetch..."
@@ -146,16 +157,19 @@ extensions will be restored.
           install_defaults.to_s['--env-shebang']
         end
 
-      installer = Gem::Installer.new(gem,
-                                     :wrappers => true,
-                                     :force => true,
-                                     :install_dir => spec.base_dir,
-                                     :env_shebang => env_shebang,
-                                     :build_args => spec.build_args)
+      installer_options = {
+        :wrappers => true,
+        :force => true,
+        :install_dir => spec.base_dir,
+        :env_shebang => env_shebang,
+        :build_args => spec.build_args,
+      }
 
       if options[:only_executables] then
+        installer = Gem::Installer.for_spec(spec, installer_options)
         installer.generate_bin
       else
+        installer = Gem::Installer.at(gem, installer_options)
         installer.install
       end
 
