@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 ##
 # Resolver sets are used to look up specifications (and their
 # dependencies) used in resolution.  This set is abstract.
@@ -20,6 +21,7 @@ class Gem::Resolver::Set
   attr_accessor :prerelease
 
   def initialize # :nodoc:
+    require 'uri'
     @prerelease = false
     @remote     = true
     @errors     = []
@@ -53,4 +55,3 @@ class Gem::Resolver::Set
   end
 
 end
-
