@@ -1,8 +1,9 @@
+# frozen_string_literal: false
 # = uri/ftp.rb
 #
 # Author:: Akira Yamada <akira@ruby-lang.org>
 # License:: You can redistribute it and/or modify it under the same term as Ruby.
-# Revision:: $Id$
+# Revision:: $Id: ftp.rb 59598 2017-08-15 01:05:50Z nobu $
 #
 # See URI for general documentation
 #
@@ -85,8 +86,8 @@ module URI
     #     require 'uri'
     #
     #     uri = URI::FTP.build(['user:password', 'ftp.example.com', nil,
-    #       '/path/file.> zip', 'i'])
-    #     puts uri.to_s  ->  ftp://user:password@ftp.example.com/%2Fpath/file.zip;type=a
+    #       '/path/file.zip', 'i'])
+    #     puts uri.to_s  ->  ftp://user:password@ftp.example.com/%2Fpath/file.zip;type=i
     #
     #     uri2 = URI::FTP.build({:host => 'ftp.example.com',
     #       :path => 'ruby/src'})
@@ -129,17 +130,24 @@ module URI
     # Arguments are +scheme+, +userinfo+, +host+, +port+, +registry+, +path+,
     # +opaque+, +query+ and +fragment+, in that order.
     #
-    def initialize(*arg)
-      raise InvalidURIError unless arg[5]
-      arg[5] = arg[5].sub(/^\//,'').sub(/^%2F/,'/')
-      super(*arg)
+    def initialize(scheme,
+                   userinfo, host, port, registry,
+                   path, opaque,
+                   query,
+                   fragment,
+                   parser = nil,
+                   arg_check = false)
+      raise InvalidURIError unless path
+      path = path.sub(/^\//,'')
+      path.sub!(/^%2F/,'/')
+      super(scheme, userinfo, host, port, registry, path, opaque,
+            query, fragment, parser, arg_check)
       @typecode = nil
-      tmp = @path.index(TYPECODE_PREFIX)
-      if tmp
+      if tmp = @path.index(TYPECODE_PREFIX)
         typecode = @path[tmp + TYPECODE_PREFIX.size..-1]
         @path = @path[0..tmp - 1]
 
-        if arg[-1]
+        if arg_check
           self.typecode = typecode
         else
           self.set_typecode(typecode)
@@ -218,7 +226,7 @@ module URI
     # RFC 1738 specifically states that the path for an FTP URI does not
     # include the / which separates the URI path from the URI host. Example:
     #
-    #     ftp://ftp.example.com/pub/ruby
+    # +ftp://ftp.example.com/pub/ruby+
     #
     # The above URI indicates that the client should connect to
     # ftp.example.com then cd pub/ruby from the initial login directory.
@@ -226,7 +234,7 @@ module URI
     # If you want to cd to an absolute directory, you must include an
     # escaped / (%2F) in the path. Example:
     #
-    #     ftp://ftp.example.com/%2Fpub/ruby
+    # +ftp://ftp.example.com/%2Fpub/ruby+
     #
     # This method will then return "/pub/ruby"
     #
