@@ -121,10 +121,10 @@ class TestDRbYield < Test::Unit::TestCase
 
   def test_06_taint
     x = proc {}
-    assert(! x.tainted?)
+    assert_not_predicate(x, :tainted?)
     @there.echo_yield(x) {|o|
       assert_equal(x, o)
-      assert(! x.tainted?)
+      assert_not_predicate(x, :tainted?)
     }
   end
 end
@@ -240,55 +240,6 @@ class TestDRbSafe1 < TestDRbAry
   end
 end
 
-class TestDRbEval # < Test::Unit::TestCase
-  def setup
-    super
-    @ext = DRbService.ext_service('ut_eval.rb')
-    @there = @ext.front
-  end
-
-  def teardown
-    @ext.stop_service if @ext
-  end
-
-  def test_01_safe1_safe4_eval
-    assert_raise(SecurityError) do
-      @there.method_missing(:instance_eval, 'ENV.inspect')
-    end
-
-    assert_raise(SecurityError) do
-      @there.method_missing(:send, :eval, 'ENV.inspect')
-    end
-
-    remote_class = @there.remote_class
-
-    assert_raise(SecurityError) do
-      remote_class.class_eval('ENV.inspect')
-    end
-
-    assert_raise(SecurityError) do
-      remote_class.module_eval('ENV.inspect')
-    end
-
-    four = @there.four
-    assert_equal(1, four.method_missing(:send, :eval, '1'))
-
-    remote_class = four.remote_class
-
-    assert_equal(1, remote_class.class_eval('1'))
-
-    assert_equal(1, remote_class.module_eval('1'))
-
-    assert_raise(SecurityError) do
-      remote_class.class_eval('ENV = {}')
-    end
-
-    assert_raise(SecurityError) do
-      remote_class.module_eval('ENV = {}')
-    end
-  end
-end
-
 class TestDRbLarge < Test::Unit::TestCase
   include DRbBase
 
@@ -306,11 +257,18 @@ class TestDRbLarge < Test::Unit::TestCase
     ary = [2] * 10240
     assert_equal(10240, @there.size(ary))
     assert_equal(20480, @there.sum(ary))
+    assert_equal(2 ** 10240, @there.multiply(ary))
+    assert_equal(2, @there.avg(ary))
+    assert_equal(2, @there.median(ary))
   end
 
   def test_02_large_ary
     ary = ["Hello, World"] * 10240
     assert_equal(10240, @there.size(ary))
+    assert_equal(ary[0..ary.length].inject(:+), @there.sum(ary))
+    assert_raise(TypeError) {@there.multiply(ary)}
+    assert_raise(TypeError) {@there.avg(ary)}
+    assert_raise(TypeError) {@there.median(ary)}
   end
 
   def test_03_large_ary
@@ -334,6 +292,41 @@ class TestDRbLarge < Test::Unit::TestCase
     end
     assert_kind_of(StandardError, exception)
   end
+
+  def test_06_array_operations
+    ary = [1,50,3,844,7,45,23]
+    assert_equal(7, @there.size(ary))
+    assert_equal(973, @there.sum(ary))
+    assert_equal(917217000, @there.multiply(ary))
+    assert_equal(139.0, @there.avg(ary))
+    assert_equal(23.0, @there.median(ary))
+
+    ary2 = [1,2,3,4]
+    assert_equal(4, @there.size(ary2))
+    assert_equal(10, @there.sum(ary2))
+    assert_equal(24, @there.multiply(ary2))
+    assert_equal(2.5, @there.avg(ary2))
+    assert_equal(2.5, @there.median(ary2))
+
+  end
+
+  def test_07_one_element_array
+    ary = [50]
+    assert_equal(1, @there.size(ary))
+    assert_equal(50, @there.sum(ary))
+    assert_equal(50, @there.multiply(ary))
+    assert_equal(50.0, @there.avg(ary))
+    assert_equal(50.0, @there.median(ary))
+  end
+
+  def test_08_empty_array
+    ary = []
+    assert_equal(0, @there.size(ary))
+    assert_equal(nil, @there.sum(ary))
+    assert_equal(nil, @there.multiply(ary))
+    assert_equal(nil, @there.avg(ary))
+    assert_equal(nil, @there.median(ary))
+  end
 end
 
 class TestBug4409 < Test::Unit::TestCase
@@ -351,7 +344,7 @@ class TestBug4409 < Test::Unit::TestCase
 
   def test_bug4409
     foo = @there.foo
-    assert(@there.foo?(foo))
+    assert_operator(@there, :foo?, foo)
   end
 end
 
