@@ -1623,15 +1623,11 @@ class TestMiniTestUnitTestCase < MiniTest::Unit::TestCase
       def test_test1; assert "does not matter" end
       def test_test2; assert "does not matter" end
       def test_test3; assert "does not matter" end
+      @test_order = [1, 0, 2]
+      def self.rand(n) @test_order.shift; end
     end
 
-    srand 42
-    expected = case
-               when maglev? then
-                 %w(test_test2 test_test3 test_test1)
-               else
-                 %w(test_test2 test_test1 test_test3)
-               end
+    expected = %w(test_test2 test_test1 test_test3)
     assert_equal expected, sample_test_case.test_methods
   end
 
@@ -1649,7 +1645,7 @@ class TestMiniTestUnitTestCase < MiniTest::Unit::TestCase
     assert_equal expected, sample_test_case.test_methods
   end
 
-  def util_assert_triggered expected, klass = MiniTest::Assertion
+  def assert_triggered expected, klass = MiniTest::Assertion
     e = assert_raises klass do
       yield
     end
@@ -1660,6 +1656,7 @@ class TestMiniTestUnitTestCase < MiniTest::Unit::TestCase
 
     assert_equal expected, msg
   end
+  alias util_assert_triggered assert_triggered
 
   def util_msg exp, act, msg = nil
     s = "Expected: #{exp.inspect}\n  Actual: #{act.inspect}"
