@@ -1,4 +1,4 @@
-# frozen_string_literal: false
+# frozen_string_literal: true
 require_relative 'helper'
 
 require 'stringio'
@@ -84,7 +84,7 @@ class TestPsych < Psych::TestCase
 
   def test_dump_io
     hash = {'hello' => 'TGIF!'}
-    stringio = StringIO.new ''
+    stringio = StringIO.new ''.dup
     assert_equal stringio, Psych.dump(hash, stringio)
     assert_equal Psych.dump(hash), stringio.string
   end
@@ -98,8 +98,8 @@ class TestPsych < Psych::TestCase
     assert_equal Psych.libyaml_version.join('.'), Psych::LIBYAML_VERSION
   end
 
-  def test_load_documents
-    docs = Psych.load_documents("--- foo\n...\n--- bar\n...")
+  def test_load_stream
+    docs = Psych.load_stream("--- foo\n...\n--- bar\n...")
     assert_equal %w{ foo bar }, docs
   end
 
@@ -121,17 +121,17 @@ class TestPsych < Psych::TestCase
 
   def test_domain_types
     got = nil
-    Psych.add_domain_type 'foo.bar,2002', 'foo' do |type, val|
+    Psych.add_domain_type 'foo.bar/2002', 'foo' do |type, val|
       got = val
     end
 
-    Psych.load('--- !foo.bar,2002/foo hello')
+    Psych.load('--- !foo.bar/2002:foo hello')
     assert_equal 'hello', got
 
-    Psych.load("--- !foo.bar,2002/foo\n- hello\n- world")
+    Psych.load("--- !foo.bar/2002:foo\n- hello\n- world")
     assert_equal %w{ hello world }, got
 
-    Psych.load("--- !foo.bar,2002/foo\nhello: world")
+    Psych.load("--- !foo.bar/2002:foo\nhello: world")
     assert_equal({ 'hello' => 'world' }, got)
   end
 
@@ -145,8 +145,9 @@ class TestPsych < Psych::TestCase
   end
 
   def test_load_file_with_fallback
-    t = Tempfile.create(['empty', 'yml'])
-    assert_equal Hash.new, Psych.load_file(t.path, Hash.new)
+    Tempfile.create(['empty', 'yml']) {|t|
+      assert_equal Hash.new, Psych.load_file(t.path, fallback: Hash.new)
+    }
   end
 
   def test_parse_file
@@ -169,16 +170,31 @@ class TestPsych < Psych::TestCase
     types = []
     appender = lambda { |*args| types << args }
 
-    Psych.add_builtin_type('foo', &appender)
-    Psych.add_domain_type('example.com,2002', 'foo', &appender)
+    Psych.add_domain_type('example.com:2002', 'foo', &appender)
     Psych.load <<-eoyml
-- !tag:yaml.org,2002:foo bar
-- !tag:example.com,2002:foo bar
+- !tag:example.com:2002:foo bar
     eoyml
 
     assert_equal [
-      ["tag:yaml.org,2002:foo", "bar"],
-      ["tag:example.com,2002:foo", "bar"]
+      ["tag:example.com:2002:foo", "bar"]
     ], types
   end
+
+  def test_symbolize_names
+    yaml = <<-eoyml
+foo:
+  bar: baz
+hoge:
+  - fuga: piyo
+    eoyml
+
+    result = Psych.load(yaml)
+    assert_equal result, { "foo" => { "bar" => "baz"}, "hoge" => [{ "fuga" => "piyo" }] }
+
+    result = Psych.load(yaml, symbolize_names: true)
+    assert_equal result, { foo: { bar: "baz" }, hoge: [{ fuga: "piyo" }] }
+
+    result = Psych.safe_load(yaml, symbolize_names: true)
+    assert_equal result, { foo: { bar: "baz" }, hoge: [{ fuga: "piyo" }] }
+  end
 end
