@@ -1,5 +1,4 @@
-# frozen_string_literal: false
-require 'rubygems'
+# frozen_string_literal: true
 require 'minitest/autorun'
 require 'pp'
 
@@ -602,7 +601,7 @@ foo
 
         para("Unordered (bulleted) lists use asterisks, pluses, and hyphens (<code>*</code>,\n" +
              "<code>+</code>, and <code>-</code>) as list markers. These three markers are\n" +
-             "interchangable; this:"),
+             "interchangeable; this:"),
 
         verb("*   Candy.\n",
              "*   Gum.\n",
@@ -1091,7 +1090,7 @@ foo
 
         para("Markdown supports ordered (numbered) and unordered (bulleted) lists."),
 
-        para("Unordered lists use asterisks, pluses, and hyphens -- interchangably\n" +
+        para("Unordered lists use asterisks, pluses, and hyphens -- interchangeably\n" +
              "-- as list markers:"),
 
         verb("*   Red\n",
@@ -1882,4 +1881,3 @@ foo
   end
 
 end
-
