@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require 'rdoc/test_case'
 
 =begin
@@ -58,7 +59,7 @@ class TestRDocParserC < RDoc::TestCase
   def teardown
     super
 
-    @tempfile.close
+    @tempfile.close!
   end
 
   def test_class_can_parse
@@ -175,6 +176,50 @@ void Init_Blah(void) {
     assert_equal 'This is a writer', writer.comment.text
   end
 
+  def test_do_attr_rb_attr_2
+    content = <<-EOF
+void Init_Blah(void) {
+  cBlah = rb_define_class("Blah", rb_cObject);
+
+  /*
+   * This is an accessor
+   */
+  rb_attr(cBlah, rb_intern_const("accessor"), 1, 1, Qfalse);
+
+  /*
+   * This is a reader
+   */
+  rb_attr(cBlah, rb_intern_const("reader"), 1, 0, Qfalse);
+
+  /*
+   * This is a writer
+   */
+  rb_attr(cBlah, rb_intern_const("writer"), 0, 1, Qfalse);
+}
+    EOF
+
+    klass = util_get_class content, 'cBlah'
+
+    attrs = klass.attributes
+    assert_equal 3, attrs.length, attrs.inspect
+
+    accessor = attrs.shift
+    assert_equal 'accessor',            accessor.name
+    assert_equal 'RW',                  accessor.rw
+    assert_equal 'This is an accessor', accessor.comment.text
+    assert_equal @top_level,            accessor.file
+
+    reader = attrs.shift
+    assert_equal 'reader',           reader.name
+    assert_equal 'R',                reader.rw
+    assert_equal 'This is a reader', reader.comment.text
+
+    writer = attrs.shift
+    assert_equal 'writer',           writer.name
+    assert_equal 'W',                writer.rw
+    assert_equal 'This is a writer', writer.comment.text
+  end
+
   def test_do_attr_rb_define_attr
     content = <<-EOF
 void Init_Blah(void) {
@@ -312,6 +357,25 @@ VALUE cFoo = rb_define_class("Foo", rb_cObject);
     assert_equal "this is the Foo class", klass.comment.text
   end
 
+  def test_do_classes_duplicate_class
+    content = <<-EOF
+/* Document-class: Foo
+ * first
+ */
+VALUE cFoo = rb_define_class("Foo", rb_cObject);
+/* Document-class: Foo
+ * second
+ */
+VALUE cFoo = rb_define_class("Foo", rb_cObject);
+    EOF
+
+    klass = util_get_class content, 'cFoo'
+    assert_equal 1, klass.comment_location.size
+    first = klass.comment_location.first
+    first_comment = first[0]
+    assert_equal 'first', first_comment.text
+  end
+
   def test_do_classes_struct
     content = <<-EOF
 /* Document-class: Foo
@@ -534,10 +598,6 @@ void Init_curses(){
   def test_do_constants_file
     content = <<-EOF
 void Init_File(void) {
-  rb_cFile = rb_define_class("File", rb_cIO);
-  rb_mFConst = rb_define_module_under(rb_cFile, "Constants");
-  rb_include_module(rb_cIO, rb_mFConst);
-
   /*  Document-const: LOCK_SH
    *
    *  Shared lock
@@ -601,6 +661,7 @@ void Init_Blah(void) {
       klass = util_get_class content, 'cDate'
     end
 
+    assert_equal 'Date', klass.full_name
     assert_match ' blah.c ', err
   end
 
@@ -623,6 +684,7 @@ void Init_Blah(void) {
       klass = util_get_class content, 'cDate'
     end
 
+    assert_equal 'Date', klass.full_name
     assert_match ' blah.cpp ', err
   end
 
@@ -645,6 +707,7 @@ void Init_Blah(void) {
       klass = util_get_class content, 'cDate'
     end
 
+    assert_equal 'Date', klass.full_name
     assert_match ' blah.y ', err
   end
 
@@ -740,6 +803,47 @@ rb_define_alias(C, "[]", "index");
     assert_equal "/*\n * comment\n */\n\n", comment.text
   end
 
+  def test_find_attr_comment_document_attr
+    parser= util_parser <<-C
+/*
+ * Document-attr: y
+ * comment
+ */
+    C
+
+    comment = parser.find_attr_comment nil, 'y'
+
+    assert_equal "/*\n * \n * comment\n */", comment.text
+  end
+
+  def test_find_attr_comment_document_attr_oneline
+    parser= util_parser <<-C
+/* Document-attr: y
+ * comment
+ */
+    C
+
+    comment = parser.find_attr_comment nil, 'y'
+
+    assert_equal "/* \n * comment\n */", comment.text
+  end
+
+  def test_find_attr_comment_document_attr_overlap
+    parser= util_parser <<-C
+/* Document-attr: x
+ * comment
+ */
+
+/* Document-attr: y
+ * comment
+ */
+    C
+
+    comment = parser.find_attr_comment nil, 'y'
+
+    assert_equal "/* \n * comment\n */", comment.text
+  end
+
   def test_find_class_comment
     @options.rdoc_include << File.dirname(__FILE__)
 
@@ -955,7 +1059,7 @@ Init_Foo(void) {
                  other_function.comment.text
     assert_equal '()', other_function.params
 
-    code = other_function.token_stream.first.text
+    code = other_function.token_stream.first[:text]
 
     assert_equal "VALUE\nother_function() {\n}", code
   end
@@ -1000,6 +1104,36 @@ init_gi_repository (void)
     assert_equal 2, klass.method_list.length
   end
 
+  def test_find_body_cast
+    content = <<-EOF
+/*
+ * a comment for other_function
+ */
+VALUE
+other_function() {
+}
+
+void
+Init_Foo(void) {
+    VALUE foo = rb_define_class("Foo", rb_cObject);
+
+    rb_define_method(foo, "my_method", (METHOD)other_function, 0);
+}
+    EOF
+
+    klass = util_get_class content, 'foo'
+    other_function = klass.method_list.first
+
+    assert_equal 'my_method', other_function.name
+    assert_equal "a comment for other_function",
+                 other_function.comment.text
+    assert_equal '()', other_function.params
+
+    code = other_function.token_stream.first[:text]
+
+    assert_equal "VALUE\nother_function() {\n}", code
+  end
+
   def test_find_body_define
     content = <<-EOF
 #define something something_else
@@ -1027,10 +1161,9 @@ Init_Foo(void) {
     assert_equal 'my_method', other_function.name
     assert_equal 'a comment for rb_other_function', other_function.comment.text
     assert_equal '()', other_function.params
-    assert_equal 118, other_function.offset
     assert_equal 8, other_function.line
 
-    code = other_function.token_stream.first.text
+    code = other_function.token_stream.first[:text]
 
     assert_equal "VALUE\nrb_other_function() {\n}", code
   end
@@ -1061,10 +1194,9 @@ Init_Foo(void) {
     assert_equal 'my_method', other_function.name
     assert_equal 'a comment for other_function', other_function.comment.text
     assert_equal '()', other_function.params
-    assert_equal 39, other_function.offset
     assert_equal 4, other_function.line
 
-    code = other_function.token_stream.first.text
+    code = other_function.token_stream.first[:text]
 
     assert_equal "#define other_function rb_other_function", code
   end
@@ -1179,6 +1311,36 @@ Init_Foo(void) {
     assert_equal "a comment for Foo#bar", bar.comment.text
   end
 
+  def test_find_body_macro
+    content = <<-EOF
+/*
+ * a comment for other_function
+ */
+DLL_LOCAL VALUE
+other_function() {
+}
+
+void
+Init_Foo(void) {
+    VALUE foo = rb_define_class("Foo", rb_cObject);
+
+    rb_define_method(foo, "my_method", other_function, 0);
+}
+    EOF
+
+    klass = util_get_class content, 'foo'
+    other_function = klass.method_list.first
+
+    assert_equal 'my_method', other_function.name
+    assert_equal "a comment for other_function",
+                 other_function.comment.text
+    assert_equal '()', other_function.params
+
+    code = other_function.token_stream.first[:text]
+
+    assert_equal "DLL_LOCAL VALUE\nother_function() {\n}", code
+  end
+
   def test_find_modifiers_call_seq
     comment = RDoc::Comment.new <<-COMMENT
 call-seq:
@@ -1260,7 +1422,6 @@ rb_m(int argc, VALUE *argv, VALUE obj) {
 
     assert_equal 'm', m.name
     assert_equal @top_level, m.file
-    assert_equal 115, m.offset
     assert_equal 7, m.line
 
     assert_equal '(p1)', m.params
@@ -1669,6 +1830,96 @@ void Init(void) {
     assert_equal expected, @store.c_singleton_class_variables
   end
 
+  def test_scan_method_copy
+    parser = util_parser <<-C
+/*
+ *  call-seq:
+ *    pathname.to_s    -> string
+ *    pathname.to_path -> string
+ *
+ *  Return the path as a String.
+ *
+ *  to_path is implemented so Pathname objects are usable with File.open, etc.
+ */
+static VALUE
+path_to_s(VALUE self) { }
+
+/*
+ *  call-seq:
+ *     str[index]               -> new_str or nil
+ *     str[start, length]       -> new_str or nil
+ *     str.slice(index)         -> new_str or nil
+ *     str.slice(start, length) -> new_str or nil
+ */
+static VALUE
+path_aref_m(int argc, VALUE *argv, VALUE str) { }
+
+/*
+ *  call-seq:
+ *     string <=> other_string   -> -1, 0, +1 or nil
+ */
+static VALUE
+path_cmp_m(VALUE str1, VALUE str2) { }
+
+/*
+ *  call-seq:
+ *     str == obj    -> true or false
+ *     str === obj   -> true or false
+ */
+VALUE
+rb_str_equal(VALUE str1, VALUE str2) { }
+
+Init_pathname()
+{
+    rb_cPathname = rb_define_class("Pathname", rb_cObject);
+
+    rb_define_method(rb_cPathname, "to_s",    path_to_s, 0);
+    rb_define_method(rb_cPathname, "to_path", path_to_s, 0);
+    rb_define_method(rb_cPathname, "[]",      path_aref_m, -1);
+    rb_define_method(rb_cPathname, "slice",   path_aref_m, -1);
+    rb_define_method(rb_cPathname, "<=>",     path_cmp_m, 1);
+    rb_define_method(rb_cPathname, "==",      rb_str_equal), 2);
+    rb_define_method(rb_cPathname, "===",     rb_str_equal), 2);
+}
+    C
+
+    parser.scan
+
+    pathname = @store.classes_hash['Pathname']
+
+    to_path = pathname.method_list.find { |m| m.name == 'to_path' }
+    assert_equal "pathname.to_path -> string", to_path.call_seq
+
+    to_s = pathname.method_list.find { |m| m.name == 'to_s' }
+    assert_equal "pathname.to_s    -> string", to_s.call_seq
+
+    index_expected = <<-EXPECTED.chomp
+str[index]               -> new_str or nil
+str[start, length]       -> new_str or nil
+    EXPECTED
+
+    index = pathname.method_list.find { |m| m.name == '[]' }
+    assert_equal index_expected, index.call_seq, '[]'
+
+    slice_expected = <<-EXPECTED.chomp
+str.slice(index)         -> new_str or nil
+str.slice(start, length) -> new_str or nil
+    EXPECTED
+
+    slice = pathname.method_list.find { |m| m.name == 'slice' }
+    assert_equal slice_expected, slice.call_seq
+
+    spaceship = pathname.method_list.find { |m| m.name == '<=>' }
+    assert_equal "string <=> other_string   -> -1, 0, +1 or nil",
+                 spaceship.call_seq
+
+    equals2 = pathname.method_list.find { |m| m.name == '==' }
+    assert_match 'str == obj', equals2.call_seq
+
+    equals3 = pathname.method_list.find { |m| m.name == '===' }
+    assert_match 'str === obj', equals3.call_seq
+  end
+
   def test_scan_order_dependent
     parser = util_parser <<-C
 void a(void) {
