@@ -1,4 +1,4 @@
-# coding: utf-8
+# frozen_string_literal: true
 
 require 'rdoc/test_case'
 
@@ -15,8 +15,6 @@ class TestRDocText < RDoc::TestCase
   end
 
   def test_self_encode_fallback
-    skip "Encoding not implemented" unless Object.const_defined? :Encoding
-
     assert_equal '…',
                  RDoc::Text::encode_fallback('…', Encoding::UTF_8,    '...')
     assert_equal '...',
@@ -56,13 +54,14 @@ class TestRDocText < RDoc::TestCase
 
     assert_equal('.               .',
                  expand_tabs(".\t\t."), 'dot tab tab dot')
+
+    assert_equal('a       a',
+                 Timeout.timeout(1) {expand_tabs("\ra\ta")}, "carriage return")
   end
 
   def test_expand_tabs_encoding
-    skip "Encoding not implemented" unless Object.const_defined? :Encoding
-
     inn = "hello\ns\tdave"
-    inn.force_encoding Encoding::BINARY
+    inn = RDoc::Encoding.change_encoding inn, Encoding::BINARY
 
     out = expand_tabs inn
 
@@ -89,8 +88,6 @@ The comments associated with
   end
 
   def test_flush_left_encoding
-    skip "Encoding not implemented" unless Object.const_defined? :Encoding
-
     text = <<-TEXT
 
   we don't worry too much.
@@ -98,7 +95,7 @@ The comments associated with
   The comments associated with
     TEXT
 
-    text.force_encoding Encoding::US_ASCII
+    text = RDoc::Encoding.change_encoding text, Encoding::US_ASCII
 
     expected = <<-EXPECTED
 
@@ -299,8 +296,6 @@ paragraph will be cut off …
   end
 
   def test_strip_hashes_encoding
-    skip "Encoding not implemented" unless Object.const_defined? :Encoding
-
     text = <<-TEXT
 ##
 # we don't worry too much.
@@ -308,7 +303,7 @@ paragraph will be cut off …
 # The comments associated with
     TEXT
 
-    text.force_encoding Encoding::CP852
+    text = RDoc::Encoding.change_encoding text, Encoding::CP852
 
     expected = <<-EXPECTED
 
@@ -334,12 +329,10 @@ paragraph will be cut off …
   end
 
   def test_strip_newlines_encoding
-    skip "Encoding not implemented" unless Object.const_defined? :Encoding
-
     assert_equal Encoding::UTF_8, ''.encoding, 'Encoding sanity check'
 
     text = " \n"
-    text.force_encoding Encoding::US_ASCII
+    text = RDoc::Encoding.change_encoding text, Encoding::US_ASCII
 
     stripped = strip_newlines text
 
@@ -385,8 +378,6 @@ paragraph will be cut off …
   end
 
   def test_strip_stars_encoding
-    skip "Encoding not implemented" unless Object.const_defined? :Encoding
-
     text = <<-TEXT
 /*
  * * we don't worry too much.
@@ -395,7 +386,7 @@ paragraph will be cut off …
  */
     TEXT
 
-    text.force_encoding Encoding::CP852
+    text = RDoc::Encoding.change_encoding text, Encoding::CP852
 
     expected = <<-EXPECTED
 
@@ -411,8 +402,6 @@ paragraph will be cut off …
   end
 
   def test_strip_stars_encoding2
-    skip "Encoding not implemented" unless Object.const_defined? :Encoding
-
     text = <<-TEXT
 /*
  * * we don't worry too much.
@@ -421,7 +410,7 @@ paragraph will be cut off …
  */
     TEXT
 
-    text.force_encoding Encoding::BINARY
+    text = RDoc::Encoding.change_encoding text, Encoding::BINARY
 
     expected = <<-EXPECTED
 
@@ -507,8 +496,6 @@ The comments associated with
   end
 
   def test_to_html_encoding
-    skip "Encoding not implemented" unless Object.const_defined? :Encoding
-
     s = '...(c)'.encode Encoding::Shift_JIS
 
     html = to_html s
@@ -551,4 +538,3 @@ The comments associated with
   end
 
 end
-
